﻿using System;
using System.Collections.Generic;
using System.Drawing;

using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 경로 데이터
    /// </summary>
    public class RouteData : IInformationData
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 데이터 응답시 이벤트 - OnDataResponse

        /// <summary>
        /// 데이터 응답시 이벤트
        /// </summary>
        public event EventHandler<RequestCompletedEventArgs> OnDataResponse;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 포인트 리스트
        /// </summary>
        private readonly List<GeoPoint> pointList = new List<GeoPoint>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 포인트 리스트 - PointList

        /// <summary>
        /// 포인트 리스트
        /// </summary>
        public List<GeoPoint> PointList
        {
            get
            {
                return this.pointList;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 경로 계산하기 - CalculateRoute(point1, point2)

        /// <summary>
        /// 경로 계산하기
        /// </summary>
        /// <param name="point1">포인트 1</param>
        /// <param name="point2">포인트 2</param>
        public void CalculateRoute(GeoPoint point1, GeoPoint point2)
        {
            CalculateRouteCore(point1, point2);

            FireOnDataResponseEvent();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 데이터 응답시 이벤트 발생시키기 - FireOnDataResponseEvent()

        /// <summary>
        /// 데이터 응답시 이벤트 발생시키기
        /// </summary>
        protected void FireOnDataResponseEvent()
        {
            OnDataResponse?.Invoke(this, CreateEventArgs());
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 이벤트 인자 생성하기 - CreateEventArgs()

        /// <summary>
        /// 이벤트 인자 생성하기
        /// </summary>
        /// <returns>요청 완료시 이벤트 인자</returns>
        private RequestCompletedEventArgs CreateEventArgs()
        {
            MapItem[] mapItemArray = new MapItem[3];

            mapItemArray[1] = new MapCallout()
            {
                Location = pointList[0],
                Text     = pointList[0].ToString()
            };

            mapItemArray[2] = new MapCallout()
            {
                Location = pointList[pointList.Count - 1],
                Text     = pointList[pointList.Count - 1].ToString()
            };

            MapPolyline polyline = new MapPolyline()
            {
                IsGeodesic  = true,
                Stroke      = Color.Red,
                StrokeWidth = 4
            };

            for(int i = 0; i < pointList.Count; i++)
            {
                polyline.Points.Add(pointList[i]);
            }

            mapItemArray[0] = polyline;

            return new RequestCompletedEventArgs(mapItemArray, null, false, null);
        }

        #endregion
        #region 경로 계산하기 (코어) - CalculateRouteCore(point1, point2)

        /// <summary>
        /// 경로 계산하기 (코어)
        /// </summary>
        /// <param name="point1">포인트 1</param>
        /// <param name="point2">포인트 2</param>
        private void CalculateRouteCore(GeoPoint point1, GeoPoint point2)
        {
            this.pointList.Clear();

            this.pointList.Add(point1);
            this.pointList.Add(point2);
        }

        #endregion
    }
}